import invariant from 'tiny-invariant';
export { default as invariant } from 'tiny-invariant';
import warning from 'tiny-warning';
export { default as warning } from 'tiny-warning';
import { RouterCore, rootRouteId, shallow, getLocationChangeInfo, pick, createControlledPromise, isNotFound, isRedirect, BaseRouteApi, BaseRoute, BaseRootRoute, trimPathRight, exactPathTest, removeTrailingSlash, deepEqual, preloadWarning } from '@tanstack/router-core';
export { PathParamError, SearchParamError, TSR_DEFERRED_PROMISE, cleanPath, componentTypes, createControlledPromise, decode, deepEqual, defaultParseSearch, defaultSerializeError, defaultStringifySearch, defer, encode, escapeJSON, functionalUpdate, getInitialRouterState, interpolatePath, isMatch, isNotFound, isPlainArray, isPlainObject, isRedirect, joinPaths, lazyFn, matchByPath, matchPathname, notFound, parsePathname, parseSearchWith, pick, redirect, removeBasepath, replaceEqualDeep, resolvePath, retainSearchParams, rootRouteId, shallow, stringifySearchWith, stripSearchParams, trimPath, trimPathLeft, trimPathRight } from '@tanstack/router-core';
export { createBrowserHistory, createHashHistory, createHistory, createMemoryHistory } from '@tanstack/history';
import * as i0 from '@angular/core';
import { ChangeDetectionStrategy, Component, assertInInjectionContext, inject, Injector, runInInjectionContext, InjectionToken, ApplicationRef, createEnvironmentInjector, afterNextRender, DestroyRef, Directive, input, ViewContainerRef, EnvironmentInjector, computed, signal, ChangeDetectorRef, untracked, ElementRef, afterRenderEffect, effect, NgZone, booleanAttribute } from '@angular/core';
import { toSignal, toObservable } from '@angular/core/rxjs-interop';
import { distinctUntilChanged, BehaviorSubject, map, combineLatest, filter, withLatestFrom, switchMap, of, catchError, throwError, pairwise, tap, Subscription } from 'rxjs';
import { DOCUMENT } from '@angular/common';
import { TanStackRouterDevtoolsCore } from '@tanstack/router-devtools-core';

function distinctUntilRefChanged() {
    return distinctUntilChanged(Object.is);
}

class DefaultNotFound {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DefaultNotFound, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.3", type: DefaultNotFound, isStandalone: true, selector: "default-not-found,DefaultNotFound", host: { styleAttribute: "display: contents;" }, ngImport: i0, template: `
    <p>Page not found</p>
  `, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DefaultNotFound, decorators: [{
            type: Component,
            args: [{
                    selector: 'default-not-found,DefaultNotFound',
                    template: `
    <p>Page not found</p>
  `,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: { style: 'display: contents;' },
                }]
        }] });

function isDevMode({ injector } = {}) {
    !injector && assertInInjectionContext(isDevMode);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const document = inject(DOCUMENT);
        const window = document.defaultView;
        return !!window && 'ng' in window;
    });
}

const ROUTER = new InjectionToken('ROUTER');
const ROUTER_STATE = new InjectionToken('ROUTER_STATE');
function injectRouter() {
    return inject(ROUTER);
}
function injectRouterState() {
    return inject(ROUTER_STATE);
}
function provideRouter(router) {
    return [
        { provide: ROUTER, useValue: router },
        {
            provide: ROUTER_STATE,
            useFactory: () => {
                const state = new BehaviorSubject(router.state);
                const appRef = inject(ApplicationRef);
                const unsub = router.__store.subscribe(({ currentVal }) => {
                    state.next(currentVal);
                });
                appRef.onDestroy(() => {
                    state.complete();
                    unsub();
                });
                return state.asObservable();
            },
        },
    ];
}
const createRouter = (options) => {
    return new NgRouter(options);
};
class NgRouter extends RouterCore {
    injectors = new Map();
    envInjectors = new Map();
    constructor(options) {
        super(options);
    }
    getRouteInjector(routeId, parent, providers = []) {
        const existingInjector = this.injectors.get(routeId);
        if (existingInjector)
            return existingInjector;
        const injector = Injector.create({
            providers,
            parent,
            name: routeId,
        });
        // cache
        this.injectors.set(routeId, injector);
        return injector;
    }
    getRouteEnvInjector(routeId, parent, providers = [], router) {
        const existingInjector = this.envInjectors.get(routeId);
        if (existingInjector)
            return existingInjector;
        let route = router.routesById[routeId] || router.routesByPath[routeId];
        // walk up the route hierarchy to build the providers
        while (route) {
            if (route.options?.providers) {
                providers.push(...route.options.providers);
            }
            const parentInjector = route.parentRoute
                ? this.envInjectors.get(route.parentRoute.id)
                : null;
            if (parentInjector) {
                parent = parentInjector;
                break;
            }
            route = route.parentRoute;
        }
        const envInjector = createEnvironmentInjector(providers, parent, routeId);
        // if routeId is rootRouteId, we'll switch existing injectors' parent to the __root__ injector
        if (routeId === rootRouteId) {
            this.envInjectors.forEach((ele) => {
                if ('parent' in ele && ele.parent === parent) {
                    ele.parent = envInjector;
                }
            });
        }
        // cache
        this.envInjectors.set(routeId, envInjector);
        return envInjector;
    }
}

function routerState$({ select, injector, equal = shallow, }) {
    !injector && assertInInjectionContext(routerState$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const rootRouterState = injectRouterState();
        if (select)
            return rootRouterState.pipe(map((s) => select(s)), distinctUntilChanged(equal));
        return rootRouterState.pipe(distinctUntilChanged(equal));
    });
}
function routerState({ select, injector, equal = shallow, } = {}) {
    !injector && assertInInjectionContext(routerState);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => toSignal(routerState$({ select, injector, equal }), { injector }));
}

class OnRendered {
    match = inject(RouteMatch);
    router = injectRouter();
    parentRouteId$ = combineLatest([
        this.match.matchId$,
        routerState$({ select: (s) => s.matches }),
    ]).pipe(map(([matchId, matches]) => matches.find((d) => d.id === matchId)?.routeId), distinctUntilRefChanged());
    location$ = routerState$({
        select: (s) => s.resolvedLocation?.state.key,
    });
    constructor() {
        let subscription;
        afterNextRender(() => {
            subscription = combineLatest([
                this.parentRouteId$,
                this.location$,
            ]).subscribe(([parentRouteId]) => {
                if (!parentRouteId || parentRouteId !== rootRouteId)
                    return;
                this.router.emit({
                    type: 'onRendered',
                    ...getLocationChangeInfo(this.router.state),
                });
            });
        });
        inject(DestroyRef).onDestroy(() => {
            subscription?.unsubscribe();
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: OnRendered, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.3", type: OnRendered, isStandalone: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: OnRendered, decorators: [{
            type: Directive
        }], ctorParameters: () => [] });
const MATCH_ID = new InjectionToken('MATCH_ID');
class RouteMatch {
    matchId = input.required();
    isDevMode = isDevMode();
    router = injectRouter();
    vcr = inject(ViewContainerRef);
    injector = inject(Injector);
    environmentInjector = inject(EnvironmentInjector);
    matchId$ = toObservable(this.matchId);
    resetKey$ = routerState$({ select: (s) => s.loadedAt.toString() });
    matches$ = routerState$({ select: (s) => s.matches });
    routeId$ = combineLatest([this.matchId$, this.matches$]).pipe(map(([matchId, matches]) => matches.find((d) => d.id === matchId)?.routeId), distinctUntilRefChanged());
    route$ = this.routeId$.pipe(map((routeId) => this.router.routesById[routeId]), distinctUntilRefChanged());
    pendingComponent$ = this.route$.pipe(map((route) => route.options.pendingComponent ||
        this.router.options.defaultPendingComponent), distinctUntilRefChanged());
    errorComponent$ = this.route$.pipe(map((route) => route.options.errorComponent ||
        this.router.options.defaultErrorComponent), distinctUntilRefChanged());
    onCatch$ = this.route$.pipe(map((route) => route.options.onCatch || this.router.options.defaultOnCatch), distinctUntilRefChanged());
    matchIndex$ = combineLatest([this.matchId$, this.matches$]).pipe(map(([matchId, matches]) => matches.findIndex((d) => d.id === matchId)), distinctUntilRefChanged());
    matchState$ = combineLatest([this.matchIndex$, this.matches$]).pipe(map(([matchIndex, matches]) => matches[matchIndex]), filter((match) => !!match), map((match) => ({
        routeId: match.routeId,
        match: pick(match, ['id', 'status', 'error']),
    })));
    matchRoute$ = this.matchState$.pipe(map(({ routeId }) => this.router.routesById[routeId]), distinctUntilRefChanged());
    match$ = this.matchState$.pipe(map(({ match }) => match), distinctUntilChanged((a, b) => !!a && !!b && a.id === b.id && a.status === b.status));
    matchLoad$ = this.match$.pipe(withLatestFrom(this.matchRoute$), switchMap(([match, matchRoute]) => {
        const loadPromise = this.router.getMatch(match.id)?.loadPromise;
        if (!loadPromise)
            return Promise.resolve();
        if (match.status === 'pending') {
            const pendingMinMs = matchRoute.options.pendingMinMs ??
                this.router.options.defaultPendingMinMs;
            let minPendingPromise = this.router.getMatch(match.id)?.minPendingPromise;
            if (pendingMinMs && !minPendingPromise) {
                // Create a promise that will resolve after the minPendingMs
                if (!this.router.isServer) {
                    minPendingPromise = createControlledPromise();
                    Promise.resolve().then(() => {
                        this.router.updateMatch(match.id, (prev) => ({
                            ...prev,
                            minPendingPromise,
                        }));
                    });
                    setTimeout(() => {
                        minPendingPromise?.resolve();
                        // We've handled the minPendingPromise, so we can delete it
                        this.router.updateMatch(match.id, (prev) => ({
                            ...prev,
                            minPendingPromise: undefined,
                        }));
                    }, pendingMinMs);
                }
            }
            return minPendingPromise?.then(() => loadPromise) || loadPromise;
        }
        return loadPromise;
    }), distinctUntilRefChanged());
    run$ = this.routeId$.pipe(switchMap((routeId) => {
        invariant(routeId, `Could not find routeId for matchId "${this.matchId()}". Please file an issue!`);
        return combineLatest([
            this.match$,
            this.matchRoute$,
            this.resetKey$,
        ]).pipe(switchMap(([match, route]) => {
            if (match.status === 'notFound') {
                invariant(isNotFound(match.error), 'Expected a notFound error');
                let notFoundCmp;
                if (!route.options.notFoundComponent) {
                    notFoundCmp = this.router.options.defaultNotFoundComponent?.();
                    if (!notFoundCmp) {
                        if (this.isDevMode) {
                            warning(route.options.notFoundComponent, `A notFoundError was encountered on the route with ID "${route.id}", but a notFoundComponent option was not configured, nor was a router level defaultNotFoundComponent configured. Consider configuring at least one of these to avoid TanStack Router's overly generic defaultNotFoundComponent (<p>Page not found</p>)`);
                        }
                        notFoundCmp = DefaultNotFound;
                    }
                }
                else {
                    notFoundCmp = route.options.notFoundComponent?.();
                }
                if (!notFoundCmp)
                    return of(null);
                const injector = this.router.getRouteInjector(route.id + '-not-found', this.injector, [{ provide: NOT_FOUND_COMPONENT_CONTEXT, useValue: {} }]);
                return of({
                    component: notFoundCmp,
                    injector,
                    environmentInjector: null,
                    clearView: true,
                });
            }
            if (match.status === 'redirected' || match.status === 'pending') {
                if (match.status === 'redirected') {
                    invariant(isRedirect(match.error), 'Expected a redirect error');
                }
                return this.matchLoad$.pipe(withLatestFrom(this.pendingComponent$), switchMap(([, pendingComponent]) => {
                    const pendingCmp = pendingComponent?.();
                    if (!pendingCmp)
                        return of(null);
                    return of({
                        component: pendingCmp,
                        injector: null,
                        environmentInjector: null,
                        clearView: true,
                    });
                }));
            }
            if (match.status === 'error') {
                return of(null).pipe(withLatestFrom(this.errorComponent$), switchMap(([, errorComponent]) => {
                    const errorCmp = errorComponent?.() || DefaultError;
                    const injector = this.router.getRouteInjector(route.id + '-error', this.injector, [
                        {
                            provide: ERROR_COMPONENT_CONTEXT,
                            useValue: {
                                error: match.error,
                                info: { componentStack: '' },
                                reset: () => void this.router.invalidate(),
                            },
                        },
                    ]);
                    return of({
                        component: errorCmp,
                        injector,
                        environmentInjector: null,
                        clearView: true,
                    });
                }));
            }
            if (match.status === 'success') {
                const successComponent = route.options.component?.() || Outlet;
                if (this.cmp === successComponent) {
                    return of({ clearView: false });
                }
                this.cmpRef = undefined;
                this.cmp = successComponent;
                const injector = this.router.getRouteInjector(route.id, this.injector);
                const environmentInjector = this.router.getRouteEnvInjector(route.id, this.environmentInjector, route.options.providers || [], this.router);
                return of({
                    component: successComponent,
                    injector: Injector.create({
                        providers: [{ provide: MATCH_ID, useValue: match.id }],
                        parent: injector,
                    }),
                    environmentInjector,
                    clearView: true,
                });
            }
            return of(null);
        }));
    }), catchError((error) => of(null).pipe(withLatestFrom(this.onCatch$), switchMap(([, onCatch]) => throwError(() => [error, onCatch])))));
    cmp;
    cmpRef;
    constructor() {
        let subscription;
        afterNextRender(() => {
            subscription = this.run$.subscribe({
                next: (runData) => {
                    if (!runData)
                        return;
                    if (!runData.clearView) {
                        this.cmpRef?.changeDetectorRef.markForCheck();
                        return;
                    }
                    const { component, injector, environmentInjector } = runData;
                    this.vcr.clear();
                    this.cmpRef = this.vcr.createComponent(component, {
                        injector: injector || undefined,
                        environmentInjector: environmentInjector || undefined,
                    });
                    this.cmpRef.changeDetectorRef.markForCheck();
                },
                error: (error) => {
                    if (Array.isArray(error)) {
                        const [errorToThrow, onCatch] = error;
                        if (onCatch)
                            onCatch(errorToThrow);
                        console.error(errorToThrow);
                        return;
                    }
                    console.error(error);
                },
            });
        });
        inject(DestroyRef).onDestroy(() => {
            subscription?.unsubscribe();
            this.vcr.clear();
            this.cmp = undefined;
            this.cmpRef = undefined;
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouteMatch, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.2.3", type: RouteMatch, isStandalone: true, selector: "route-match,RouteMatch", inputs: { matchId: { classPropertyName: "matchId", publicName: "matchId", isSignal: true, isRequired: true, transformFunction: null } }, host: { properties: { "attr.data-matchId": "matchId()" } }, hostDirectives: [{ directive: OnRendered }], ngImport: i0, template: ``, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouteMatch, decorators: [{
            type: Component,
            args: [{
                    selector: 'route-match,RouteMatch',
                    template: ``,
                    hostDirectives: [OnRendered],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: {
                        '[attr.data-matchId]': 'matchId()',
                    },
                }]
        }], ctorParameters: () => [] });
class Outlet {
    matchId = inject(MATCH_ID);
    router = injectRouter();
    vcr = inject(ViewContainerRef);
    isDevMode = isDevMode();
    defaultPendingComponent = this.router.options.defaultPendingComponent?.();
    matches$ = routerState$({ select: (s) => s.matches });
    routeId$ = this.matches$.pipe(map((matches) => matches.find((d) => d.id === this.matchId)?.routeId), distinctUntilRefChanged());
    route$ = this.routeId$.pipe(map((routeId) => this.router.routesById[routeId]), distinctUntilRefChanged());
    parentGlobalNotFound$ = this.matches$.pipe(map((matches) => {
        const parentMatch = matches.find((d) => d.id === this.matchId);
        if (!parentMatch) {
            warning(false, `Could not find parent match for matchId "${this.matchId}". Please file an issue!`);
            return false;
        }
        return parentMatch.globalNotFound;
    }));
    childMatchId$ = this.matches$.pipe(map((matches) => {
        const index = matches.findIndex((d) => d.id === this.matchId);
        if (index === -1)
            return null;
        return matches[index + 1]?.id;
    }), distinctUntilRefChanged());
    matchLoad$ = this.childMatchId$.pipe(switchMap((childMatchId) => {
        if (!childMatchId)
            return Promise.resolve();
        const loadPromise = this.router.getMatch(childMatchId)?.loadPromise;
        if (!loadPromise)
            return Promise.resolve();
        return loadPromise;
    }));
    renderedId;
    cmpRef;
    run$ = combineLatest([
        this.parentGlobalNotFound$,
        this.childMatchId$,
    ]).pipe(switchMap(([parentGlobalNotFound, childMatchId]) => {
        if (parentGlobalNotFound) {
            return this.route$.pipe(map((route) => {
                let notFoundCmp = undefined;
                if (!route.options.notFoundComponent) {
                    notFoundCmp = this.router.options.defaultNotFoundComponent?.();
                    if (!notFoundCmp) {
                        if (this.isDevMode) {
                            warning(route.options.notFoundComponent, `A notFoundError was encountered on the route with ID "${route.id}", but a notFoundComponent option was not configured, nor was a router level defaultNotFoundComponent configured. Consider configuring at least one of these to avoid TanStack Router's overly generic defaultNotFoundComponent (<p>Page not found</p>)`);
                        }
                        notFoundCmp = DefaultNotFound;
                    }
                }
                else {
                    notFoundCmp = route.options.notFoundComponent?.();
                }
                if (!notFoundCmp)
                    return null;
                this.renderedId = route.id + '-not-found';
                const injector = this.router.getRouteInjector(route.id + '-not-found', this.vcr.injector, [{ provide: NOT_FOUND_COMPONENT_CONTEXT, useValue: {} }]);
                return {
                    component: notFoundCmp,
                    injector,
                    clearView: true,
                    childMatchId: null,
                };
            }));
        }
        if (!childMatchId)
            return of(null);
        if (this.renderedId === childMatchId) {
            return of({ clearView: false });
        }
        this.cmpRef = undefined;
        if (childMatchId === rootRouteId) {
            return this.matchLoad$.pipe(map(() => {
                return {
                    component: this.defaultPendingComponent,
                    injector: null,
                    clearView: true,
                    childMatchId: null,
                };
            }));
        }
        this.renderedId = childMatchId;
        return of({
            component: RouteMatch,
            injector: null,
            clearView: true,
            childMatchId,
        });
    }), catchError((error) => throwError(() => error)));
    constructor() {
        let subscription;
        afterNextRender(() => {
            subscription = this.run$.subscribe({
                next: (runData) => {
                    if (!runData)
                        return;
                    if (!runData.clearView) {
                        this.cmpRef?.changeDetectorRef.markForCheck();
                        return;
                    }
                    const { component, injector, childMatchId } = runData;
                    this.vcr.clear();
                    if (!component)
                        return;
                    this.cmpRef = this.vcr.createComponent(component, {
                        injector: injector || undefined,
                    });
                    if (childMatchId) {
                        this.cmpRef.setInput('matchId', childMatchId);
                    }
                    this.cmpRef.changeDetectorRef.markForCheck();
                },
                error: (error) => {
                    console.error(error);
                },
            });
        });
        inject(DestroyRef).onDestroy(() => {
            subscription?.unsubscribe();
            this.vcr.clear();
            this.cmpRef = undefined;
            this.renderedId = undefined;
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Outlet, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.3", type: Outlet, isStandalone: true, selector: "outlet,Outlet", ngImport: i0, template: ``, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Outlet, decorators: [{
            type: Component,
            args: [{
                    selector: 'outlet,Outlet',
                    template: ``,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

function match$({ injector, ...opts }) {
    !injector && assertInInjectionContext(match$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const closestMatchId = inject(MATCH_ID, { optional: true });
        const nearestMatchId = computed(() => {
            if (opts.from)
                return null;
            return closestMatchId;
        });
        return combineLatest([
            routerState$({ select: (s) => s.matches, injector }),
            toObservable(nearestMatchId),
        ]).pipe(map(([matches, matchId]) => {
            const match = matches.find((d) => {
                return opts.from ? opts.from === d.routeId : d.id === matchId;
            });
            invariant(!((opts.shouldThrow ?? true) && !match), `Could not find ${opts.from ? `an active match from "${opts.from}"` : 'a nearest match!'}`);
            if (match === undefined) {
                return undefined;
            }
            return opts.select ? opts.select(match) : match;
        }));
    });
}
function match({ injector, ...opts }) {
    !injector && assertInInjectionContext(match);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(match$({ injector, ...opts }), { injector });
    });
}

function loaderData$({ injector, ...opts }) {
    !injector && assertInInjectionContext(loaderData$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return match$({
            injector,
            from: opts.from,
            strict: opts.strict,
            select: (s) => (opts.select ? opts.select(s.loaderData) : s.loaderData),
        });
    });
}
function loaderData({ injector, ...opts }) {
    !injector && assertInInjectionContext(loaderData);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(loaderData$({ injector, ...opts }), { injector });
    });
}

function loaderDeps$({ injector, ...opts }) {
    !injector && assertInInjectionContext(loaderDeps$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const { select, ...rest } = opts;
        return match$({
            ...rest,
            select: (s) => {
                return select ? select(s.loaderDeps) : s.loaderDeps;
            },
        });
    });
}
function loaderDeps({ injector, ...opts }) {
    !injector && assertInInjectionContext(loaderDeps);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(loaderDeps$({ injector, ...opts }), { injector });
    });
}

function params$({ injector, ...opts }) {
    !injector && assertInInjectionContext(params);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return match$({
            from: opts.from,
            strict: opts.strict,
            shouldThrow: opts.shouldThrow,
            select: (match) => {
                return opts.select ? opts.select(match.params) : match.params;
            },
        });
    });
}
function params({ injector, ...opts }) {
    !injector && assertInInjectionContext(params);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(params$({ injector, ...opts }));
    });
}

function routeContext$({ injector, ...opts }) {
    !injector && assertInInjectionContext(routeContext);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return match$({
            ...opts,
            select: (match) => {
                return opts.select ? opts.select(match.context) : match.context;
            },
        });
    });
}
function routeContext({ injector, ...opts }) {
    !injector && assertInInjectionContext(routeContext);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(routeContext$({ injector, ...opts }));
    });
}

function search$({ injector, ...opts }) {
    !injector && assertInInjectionContext(search);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return match$({
            from: opts.from,
            strict: opts.strict,
            shouldThrow: opts.shouldThrow,
            select: (match) => {
                return opts.select ? opts.select(match.search) : match.search;
            },
        });
    });
}
function search({ injector, ...opts }) {
    !injector && assertInInjectionContext(search);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(search$({ injector, ...opts }));
    });
}

const ERROR_COMPONENT_CONTEXT = new InjectionToken('ERROR_COMPONENT_CONTEXT');
const NOT_FOUND_COMPONENT_CONTEXT = new InjectionToken('NOT_FOUND_COMPONENT_CONTEXT');
function routeApi(id) {
    return new RouteApi({ id });
}
class RouteApi extends BaseRouteApi {
    /**
     * @deprecated Use the `getRouteApi` function instead.
     */
    constructor({ id }) {
        super({ id });
    }
    match$ = (opts) => match$({ ...opts, from: this.id });
    match = (opts) => match({ ...opts, from: this.id });
    routeContext$ = (opts) => routeContext$({ ...opts, from: this.id });
    routeContext = (opts) => routeContext({ ...opts, from: this.id });
    search$ = (opts) => search$({ ...opts, from: this.id });
    search = (opts) => search({ ...opts, from: this.id });
    params$ = (opts) => params$({ ...opts, from: this.id });
    params = (opts) => params({ ...opts, from: this.id });
    loaderDeps$ = (opts) => loaderDeps$({ ...opts, from: this.id });
    loaderDeps = (opts) => loaderDeps({ ...opts, from: this.id, strict: false });
    loaderData$ = (opts) => loaderData$({ ...opts, from: this.id });
    loaderData = (opts) => loaderData({ ...opts, from: this.id, strict: false });
}
class Route extends BaseRoute {
    /**
     * @deprecated Use the `createRoute` function instead.
     */
    constructor(options) {
        super(options);
    }
    match$ = (opts) => match$({ ...opts, from: this.id });
    match = (opts) => match({ ...opts, from: this.id });
    routeContext$ = (opts) => routeContext$({ ...opts, from: this.id });
    routeContext = (opts) => routeContext({ ...opts, from: this.id });
    search$ = (opts) => search$({ ...opts, from: this.id });
    search = (opts) => search({ ...opts, from: this.id });
    params$ = (opts) => params$({ ...opts, from: this.id });
    params = (opts) => params({ ...opts, from: this.id });
    loaderDeps$ = (opts) => loaderDeps$({ ...opts, from: this.id });
    loaderDeps = (opts) => loaderDeps({ ...opts, from: this.id });
    loaderData$ = (opts) => loaderData$({ ...opts, from: this.id });
    loaderData = (opts) => loaderData({ ...opts, from: this.id });
}
function createRoute(options) {
    if (options.loader) {
        options.loader = runFnInInjectionContext(options.loader);
    }
    if (options.shouldReload && typeof options.shouldReload === 'function') {
        options.shouldReload = runFnInInjectionContext(options.shouldReload);
    }
    if (options.beforeLoad) {
        options.beforeLoad = runFnInInjectionContext(options.beforeLoad);
    }
    return new Route(options);
}
function createRootRouteWithContext() {
    return (options) => {
        return createRootRoute(options);
    };
}
class RootRoute extends BaseRootRoute {
    /**
     * @deprecated `RootRoute` is now an internal implementation detail. Use `createRootRoute()` instead.
     */
    constructor(options) {
        super(options);
    }
    match$ = (opts) => match$({ ...opts, from: this.id });
    match = (opts) => match({ ...opts, from: this.id });
    routeContext$ = (opts) => routeContext$({ ...opts, from: this.id });
    routeContext = (opts) => routeContext({ ...opts, from: this.id });
    search$ = (opts) => search$({ ...opts, from: this.id });
    search = (opts) => search({ ...opts, from: this.id });
    params$ = (opts) => params$({ ...opts, from: this.id });
    params = (opts) => params({ ...opts, from: this.id });
    loaderDeps$ = (opts) => loaderDeps$({ ...opts, from: this.id });
    loaderDeps = (opts) => loaderDeps({ ...opts, from: this.id });
    loaderData$ = (opts) => loaderData$({ ...opts, from: this.id });
    loaderData = (opts) => loaderData({ ...opts, from: this.id });
}
function createRootRoute(options) {
    return new RootRoute(options);
}
class NotFoundRoute extends Route {
    constructor(options) {
        super({
            ...options,
            id: '404',
        });
    }
}
function runFnInInjectionContext(fn) {
    const originalFn = fn;
    return (...args) => {
        const { context, location, route } = args[0];
        const routeInjector = context.getRouteInjector(route?.id || location.href);
        return runInInjectionContext(routeInjector, originalFn.bind(null, ...args));
    };
}

class DefaultError {
    context = inject(ERROR_COMPONENT_CONTEXT);
    show = signal(false);
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DefaultError, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.2.3", type: DefaultError, isStandalone: true, selector: "default-error,DefaultError", ngImport: i0, template: `
    <div style="display: flex; align-items: center; gap: 0.5rem">
      <strong style="font-size: 1rem">Something went wrong!</strong>
      <button
        style="appearance: none; font-size: 0.6em; border: 1px solid currentColor; padding: 0.1rem 0.2rem; font-weight: bold; border-radius: 0.25rem"
        (click)="show.set(!show())"
      >
        {{ show() ? 'Hide Error' : 'Show Error' }}
      </button>
    </div>
    <div style="height: 0.25rem"></div>
    @if (show()) {
      <div>
        <pre
          style="font-size: 0.7em; border: 1px solid red; border-radius: 0.25rem; padding: 0.3rem; color: red; overflow: auto"
        >
          @if (context.error.message; as message) {
            <code>{{ message }}</code>
          }
        </pre>
      </div>
    }
  `, isInline: true, styles: [":host{display:block;padding:.5rem;max-width:100%}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DefaultError, decorators: [{
            type: Component,
            args: [{ selector: 'default-error,DefaultError', template: `
    <div style="display: flex; align-items: center; gap: 0.5rem">
      <strong style="font-size: 1rem">Something went wrong!</strong>
      <button
        style="appearance: none; font-size: 0.6em; border: 1px solid currentColor; padding: 0.1rem 0.2rem; font-weight: bold; border-radius: 0.25rem"
        (click)="show.set(!show())"
      >
        {{ show() ? 'Hide Error' : 'Show Error' }}
      </button>
    </div>
    <div style="height: 0.25rem"></div>
    @if (show()) {
      <div>
        <pre
          style="font-size: 0.7em; border: 1px solid red; border-radius: 0.25rem; padding: 0.3rem; color: red; overflow: auto"
        >
          @if (context.error.message; as message) {
            <code>{{ message }}</code>
          }
        </pre>
      </div>
    }
  `, changeDetection: ChangeDetectionStrategy.OnPush, styles: [":host{display:block;padding:.5rem;max-width:100%}\n"] }]
        }] });

class Transitioner {
    router = injectRouter();
    destroyRef = inject(DestroyRef);
    document = inject(DOCUMENT);
    cdr = inject(ChangeDetectorRef);
    matches$ = routerState$({ select: (s) => s.matches });
    hasPendingMatches$ = this.matches$.pipe(map((matches) => matches.some((d) => d.status === 'pending')), distinctUntilChanged(() => false));
    isLoading$ = routerState$({
        select: (s) => s.isLoading,
        equal: () => false,
    });
    previousIsLoading$ = this.isLoading$.pipe(pairwise(), map(([prev, curr]) => prev ?? curr));
    isTransitioning$ = new BehaviorSubject(false);
    isAnyPending$ = combineLatest([
        this.isLoading$,
        this.isTransitioning$,
        this.hasPendingMatches$,
    ]).pipe(map(([isLoading, isTransitioning, hasPendingMatches]) => isLoading || isTransitioning || hasPendingMatches), distinctUntilChanged(() => false));
    previousIsAnyPending$ = this.isAnyPending$.pipe(pairwise(), map(([prev, curr]) => prev ?? curr));
    isPagePending$ = combineLatest([
        this.isLoading$,
        this.hasPendingMatches$,
    ]).pipe(map(([isLoading, hasPendingMatches]) => isLoading || hasPendingMatches), distinctUntilChanged(() => false));
    previousIsPagePending$ = this.isPagePending$.pipe(pairwise(), map(([prev, curr]) => prev ?? curr));
    mountLoadForRouter = { router: this.router, mounted: false };
    load$ = combineLatest([
        this.previousIsLoading$,
        this.isLoading$,
    ]).pipe(tap(([previousIsLoading, isLoading]) => {
        if (previousIsLoading && !isLoading) {
            this.router.emit({
                type: 'onLoad',
                ...getLocationChangeInfo(this.router.state),
            });
            this.router.__store.setState((s) => ({ ...s, status: 'idle' }));
        }
    }));
    pagePending$ = combineLatest([
        this.previousIsPagePending$,
        this.isPagePending$,
    ]).pipe(tap(([previousIsPagePending, isPagePending]) => {
        // emit onBeforeRouteMount
        if (previousIsPagePending && !isPagePending) {
            this.router.emit({
                type: 'onBeforeRouteMount',
                ...getLocationChangeInfo(this.router.state),
            });
        }
    }));
    pending$ = combineLatest([
        this.previousIsAnyPending$,
        this.isAnyPending$,
    ]).pipe(tap(([previousIsAnyPending, isAnyPending]) => {
        // The router was pending and now it's not
        if (previousIsAnyPending && !isAnyPending) {
            this.router.emit({
                type: 'onResolved',
                ...getLocationChangeInfo(this.router.state),
            });
            this.router.__store.setState((s) => ({
                ...s,
                status: 'idle',
                resolvedLocation: s.location,
            }));
            if (typeof this.document !== 'undefined' &&
                'querySelector' in this.document) {
                const hashScrollIntoViewOptions = this.router.state.location.state.__hashScrollIntoViewOptions ??
                    true;
                if (hashScrollIntoViewOptions &&
                    this.router.state.location.hash !== '') {
                    const el = this.document.getElementById(this.router.state.location.hash);
                    if (el)
                        el.scrollIntoView(hashScrollIntoViewOptions);
                }
            }
        }
    }));
    constructor() {
        if (!this.router.isServer) {
            this.router.startTransition = (fn) => {
                this.isTransitioning$.next(true);
                fn();
                this.isTransitioning$.next(false);
                this.cdr.detectChanges();
            };
        }
        const subscription = new Subscription();
        // Try to load the initial location
        afterNextRender(() => {
            untracked(() => {
                const window = this.document.defaultView;
                if ((typeof window !== 'undefined' && this.router.clientSsr) ||
                    (this.mountLoadForRouter.router === this.router &&
                        this.mountLoadForRouter.mounted)) {
                    return;
                }
                this.mountLoadForRouter = { router: this.router, mounted: true };
                const tryLoad = async () => {
                    try {
                        await this.router.load();
                        this.router.__store.setState((s) => ({ ...s, status: 'idle' }));
                    }
                    catch (err) {
                        console.error(err);
                    }
                };
                void tryLoad();
            });
            subscription.add(this.load$.subscribe());
            subscription.add(this.pagePending$.subscribe());
            subscription.add(this.pending$.subscribe());
        });
        this.destroyRef.onDestroy(() => subscription.unsubscribe());
    }
    ngOnInit() {
        // Subscribe to location changes
        // and try to load the new location
        const unsub = this.router.history.subscribe(() => this.router.load());
        const nextLocation = this.router.buildLocation({
            to: this.router.latestLocation.pathname,
            search: true,
            params: true,
            hash: true,
            state: true,
            _includeValidateSearch: true,
        });
        if (trimPathRight(this.router.latestLocation.href) !==
            trimPathRight(nextLocation.href)) {
            void this.router.commitLocation({ ...nextLocation, replace: true });
        }
        this.destroyRef.onDestroy(() => unsub());
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Transitioner, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.3", type: Transitioner, isStandalone: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Transitioner, decorators: [{
            type: Directive
        }], ctorParameters: () => [] });

class Matches {
    router = injectRouter();
    injector = inject(Injector);
    vcr = inject(ViewContainerRef);
    defaultPendingComponent = this.router.options.defaultPendingComponent?.();
    resetKey$ = routerState$({ select: (s) => s.loadedAt.toString() });
    rootMatchId$ = routerState$({ select: (s) => s.matches[0]?.id });
    matchLoad$ = this.rootMatchId$.pipe(switchMap((rootMatchId) => {
        if (!rootMatchId)
            return of({ pending: false });
        const loadPromise = this.router.getMatch(rootMatchId)?.loadPromise;
        if (!loadPromise)
            return of({ pending: false });
        return of({ pending: true }).pipe(switchMap(() => loadPromise.then(() => ({ pending: false }))));
    }));
    cmpRef;
    run$ = this.matchLoad$.pipe(switchMap(({ pending }) => {
        if (pending) {
            if (this.defaultPendingComponent) {
                return of({
                    component: this.defaultPendingComponent,
                    clearView: true,
                    matchId: null,
                });
            }
            return of(null);
        }
        return combineLatest([this.rootMatchId$, this.resetKey$]).pipe(map(([matchId]) => {
            if (!matchId)
                return null;
            if (this.cmpRef)
                return { clearView: false };
            return {
                component: RouteMatch,
                matchId,
                clearView: true,
            };
        }));
    }));
    constructor() {
        let subscription;
        afterNextRender(() => {
            subscription = this.run$.subscribe({
                next: (runData) => {
                    if (!runData)
                        return;
                    if (!runData.clearView) {
                        this.cmpRef?.changeDetectorRef.markForCheck();
                        return;
                    }
                    const { component, matchId } = runData;
                    this.vcr.clear();
                    this.cmpRef = this.vcr.createComponent(component);
                    if (matchId) {
                        this.cmpRef.setInput('matchId', matchId);
                    }
                    this.cmpRef.changeDetectorRef.markForCheck();
                },
                error: (error) => {
                    console.error(error);
                    const injector = Injector.create({
                        providers: [
                            {
                                provide: ERROR_COMPONENT_CONTEXT,
                                useValue: {
                                    error: error,
                                    info: { componentStack: '' },
                                    reset: () => void this.router.invalidate(),
                                },
                            },
                        ],
                        parent: this.injector,
                    });
                    this.vcr.clear();
                    const ref = this.vcr.createComponent(DefaultError, { injector });
                    ref.changeDetectorRef.markForCheck();
                    this.cmpRef = undefined;
                },
            });
        });
        inject(DestroyRef).onDestroy(() => {
            subscription?.unsubscribe();
            this.vcr.clear();
            this.cmpRef = undefined;
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Matches, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.3", type: Matches, isStandalone: true, hostDirectives: [{ directive: Transitioner }], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Matches, decorators: [{
            type: Directive,
            args: [{ hostDirectives: [Transitioner] }]
        }], ctorParameters: () => [] });
function matches$({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(matches$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return routerState$({
            injector,
            select: (state) => {
                const matches = state.matches;
                return opts.select
                    ? opts.select(matches)
                    : matches;
            },
        });
    });
}
function matches({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(matches);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(matches$({ injector, ...opts }));
    });
}
function parentMatches$({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(parentMatches$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const closestMatch = inject(MATCH_ID);
        return routerState$({ injector, select: (s) => s.matches }).pipe(map((matches) => {
            const sliced = matches.slice(0, matches.findIndex((d) => d.id === closestMatch));
            return opts.select
                ? opts.select(sliced)
                : sliced;
        }), distinctUntilRefChanged());
    });
}
function parentMatches({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(parentMatches);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(parentMatches$({ injector, ...opts }));
    });
}
function childMatches$({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(childMatches$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const closestMatch = inject(MATCH_ID);
        return routerState$({ injector, select: (s) => s.matches }).pipe(map((matches) => {
            const sliced = matches.slice(matches.findIndex((d) => d.id === closestMatch) + 1);
            return opts.select
                ? opts.select(sliced)
                : sliced;
        }), distinctUntilRefChanged());
    });
}
function childMatches({ injector, ...opts } = {}) {
    !injector && assertInInjectionContext(childMatches);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(childMatches$({ injector, ...opts }));
    });
}

class Link {
    linkOptions = input.required({
        alias: 'link',
        transform: (value) => {
            return (typeof value === 'object' ? value : { to: value });
        },
    });
    linkActiveOptions = input({ class: 'active' }, {
        alias: 'linkActive',
        transform: (value) => {
            if (typeof value === 'string')
                return { class: value };
            if (!value.class)
                value.class = 'active';
            return value;
        },
    });
    router = injectRouter();
    hostElement = inject(ElementRef);
    currentSearch = routerState({ select: (s) => s.location.searchStr });
    disabled = computed(() => this.linkOptions()['disabled']);
    to = computed(() => this.linkOptions()['to']);
    userFrom = computed(() => this.linkOptions()['from']);
    userReloadDocument = computed(() => this.linkOptions()['reloadDocument']);
    userPreload = computed(() => this.linkOptions()['preload']);
    userPreloadDelay = computed(() => this.linkOptions()['preloadDelay']);
    activeOptions = computed(() => this.linkOptions().activeOptions || this.linkActiveOptions() || {});
    exactActiveOptions = computed(() => this.activeOptions().exact);
    includeHashActiveOptions = computed(() => this.activeOptions().includeHash);
    includeSearchActiveOptions = computed(() => this.activeOptions().includeSearch);
    type = computed(() => {
        const to = this.to();
        try {
            new URL(`${to}`);
            return 'external';
        }
        catch {
            return 'internal';
        }
    });
    // when `from` is not supplied, use the leaf route of the current matches as the `from` location
    // so relative routing works as expected
    from = toSignal(combineLatest([
        toObservable(this.userFrom),
        matches$({ select: (matches) => matches[matches.length - 1]?.fullPath }),
    ]).pipe(map(([userFrom, from]) => userFrom ?? from), distinctUntilRefChanged()));
    navigateOptions = computed(() => {
        return { ...this.linkOptions(), from: this.from() };
    });
    next = computed(() => {
        const [options] = [this.navigateOptions(), this.currentSearch()];
        try {
            return this.router.buildLocation(options);
        }
        catch (err) {
            return null;
        }
    });
    preload = computed(() => {
        const userReloadDocument = this.userReloadDocument();
        if (userReloadDocument)
            return false;
        const userPreload = this.userPreload();
        if (userPreload)
            return userPreload;
        return this.router.options.defaultPreload;
    });
    preloadDelay = computed(() => {
        const userPreloadDelay = this.userPreloadDelay();
        if (userPreloadDelay)
            return userPreloadDelay;
        return this.router.options.defaultPreloadDelay;
    });
    hostHref = computed(() => {
        const [type, to] = [this.type(), this.to()];
        if (type === 'external')
            return to;
        const disabled = this.disabled();
        if (disabled)
            return undefined;
        const next = this.next();
        if (!next)
            return undefined;
        return next.maskedLocation
            ? this.router.history.createHref(next.maskedLocation.href)
            : this.router.history.createHref(next.href);
    });
    transitioning = signal(false);
    isActive = toSignal(combineLatest([
        toObservable(this.next),
        toObservable(this.exactActiveOptions),
        toObservable(this.includeSearchActiveOptions),
        toObservable(this.includeHashActiveOptions),
        routerState$({ select: (s) => s.location }),
    ]).pipe(map(([next, exact, includeSearchOptions, includeHashOptions, location]) => {
        if (!next)
            return false;
        if (exact) {
            const testExact = exactPathTest(location.pathname, next.pathname, this.router.basepath);
            if (!testExact)
                return false;
        }
        else {
            const currentPathSplit = removeTrailingSlash(location.pathname, this.router.basepath).split('/');
            const nextPathSplit = removeTrailingSlash(next.pathname, this.router.basepath).split('/');
            const pathIsFuzzyEqual = nextPathSplit.every((d, i) => d === currentPathSplit[i]);
            if (!pathIsFuzzyEqual) {
                return false;
            }
        }
        const includeSearch = includeSearchOptions ?? true;
        if (includeSearch) {
            const searchTest = deepEqual(location.search, next.search, {
                partial: !exact,
                ignoreUndefined: !(includeSearchOptions ?? true),
            });
            if (!searchTest) {
                return false;
            }
        }
        const includeHash = includeHashOptions ?? true;
        if (includeHash) {
            return location.hash === next.hash;
        }
        return true;
    })));
    activeClass = computed(() => this.isActive() ? this.activeOptions().class || 'active' : '');
    constructor() {
        afterRenderEffect(() => {
            const [disabled, preload] = [
                untracked(this.disabled),
                untracked(this.preload),
            ];
            if (!disabled && preload === 'render') {
                this.doPreload();
            }
        });
        afterRenderEffect((onCleanup) => {
            const unsub = this.router.subscribe('onResolved', () => {
                this.transitioning.set(false);
            });
            onCleanup(() => unsub());
        });
    }
    handleClick(event) {
        if (this.type() === 'external')
            return;
        const [disabled, target] = [
            this.disabled(),
            this.hostElement.nativeElement.target,
        ];
        if (disabled ||
            this.isCtrlEvent(event) ||
            event.defaultPrevented ||
            (target && target !== '_self') ||
            event.button !== 0) {
            return;
        }
        event.preventDefault();
        this.transitioning.set(true);
        this.router.navigate(this.navigateOptions());
    }
    handleFocus() {
        if (this.disabled() || this.type() === 'external')
            return;
        if (this.preload()) {
            this.doPreload();
        }
    }
    preloadTimeout = null;
    handleMouseEnter(event) {
        if (this.disabled() ||
            !this.preload() ||
            this.isActive() ||
            this.type() === 'external')
            return;
        this.preloadTimeout = setTimeout(() => {
            this.preloadTimeout = null;
            this.doPreload();
        }, this.preloadDelay());
    }
    handleMouseLeave() {
        if (this.disabled() || this.type() === 'external')
            return;
        if (this.preloadTimeout) {
            clearTimeout(this.preloadTimeout);
            this.preloadTimeout = null;
        }
    }
    doPreload() {
        this.router.preloadRoute(this.navigateOptions()).catch((err) => {
            console.warn(err);
            console.warn(preloadWarning);
        });
    }
    isCtrlEvent(e) {
        return e.metaKey || e.altKey || e.ctrlKey || e.shiftKey;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Link, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.3", type: Link, isStandalone: true, selector: "a[link]", inputs: { linkOptions: { classPropertyName: "linkOptions", publicName: "link", isSignal: true, isRequired: true, transformFunction: null }, linkActiveOptions: { classPropertyName: "linkActiveOptions", publicName: "linkActive", isSignal: true, isRequired: false, transformFunction: null } }, host: { listeners: { "click": "handleClick($event)", "focus": "handleFocus()", "touchstart": "handleClick($event)", "mouseenter": "handleMouseEnter($event)", "mouseleave": "handleMouseLeave()" }, properties: { "class": "activeClass()", "attr.data-active": "isActive()", "attr.data-type": "type()", "attr.data-transitioning": "transitioning() ? \"transitioning\" : undefined", "attr.href": "hostHref()", "attr.role": "disabled() ? \"link\" : undefined", "attr.aria-disabled": "disabled()", "attr.aria-current": "isActive() ? \"page\" : undefined", "attr.data-from": "from()" } }, exportAs: ["link"], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: Link, decorators: [{
            type: Directive,
            args: [{
                    selector: 'a[link]',
                    exportAs: 'link',
                    host: {
                        '(click)': 'handleClick($event)',
                        '(focus)': 'handleFocus()',
                        '(touchstart)': 'handleClick($event)',
                        '(mouseenter)': 'handleMouseEnter($event)',
                        '(mouseleave)': 'handleMouseLeave()',
                        '[class]': 'activeClass()',
                        '[attr.data-active]': 'isActive()',
                        '[attr.data-type]': 'type()',
                        '[attr.data-transitioning]': 'transitioning() ? "transitioning" : undefined',
                        '[attr.href]': 'hostHref()',
                        '[attr.role]': 'disabled() ? "link" : undefined',
                        '[attr.aria-disabled]': 'disabled()',
                        '[attr.aria-current]': 'isActive() ? "page" : undefined',
                        '[attr.data-from]': 'from()',
                    },
                }]
        }], ctorParameters: () => [] });
const linkOptions = (options) => {
    return options;
};

function matchRoute$({ injector, } = {}) {
    !injector && assertInInjectionContext(matchRoute$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const router = injectRouter();
        const status$ = routerState$({ select: (s) => s.status });
        return (opts) => {
            const { pending, caseSensitive, fuzzy, includeSearch, ...rest } = opts;
            return status$.pipe(map(() => router.matchRoute(rest, {
                pending,
                caseSensitive,
                fuzzy,
                includeSearch,
            })));
        };
    });
}
function matchRoute({ injector, } = {}) {
    !injector && assertInInjectionContext(matchRoute);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        const matchRoute$Return = matchRoute$({ injector });
        return (opts) => {
            return toSignal(matchRoute$Return(opts), { injector });
        };
    });
}
class MatchRoute {
    matchRoute = input({}, { alias: 'match' });
    status$ = routerState$({ select: (s) => s.status });
    matchRouteFn = matchRoute$();
    parentLink = inject(Link, { optional: true });
    options = computed(() => {
        const parentLinkOptions = this.parentLink?.linkOptions();
        if (!parentLinkOptions)
            return this.matchRoute();
        return { ...parentLinkOptions, ...this.matchRoute() };
    });
    match$ = combineLatest([toObservable(this.options), this.status$]).pipe(switchMap(([matchRoute]) => this.matchRouteFn(matchRoute)));
    match = toSignal(this.match$);
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: MatchRoute, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.3", type: MatchRoute, isStandalone: true, selector: "[match]", inputs: { matchRoute: { classPropertyName: "matchRoute", publicName: "match", isSignal: true, isRequired: false, transformFunction: null } }, exportAs: ["matchRoute"], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: MatchRoute, decorators: [{
            type: Directive,
            args: [{ selector: '[match]', exportAs: 'matchRoute' }]
        }] });

function createFileRoute(path) {
    return new FileRoute(path, {
        silent: true,
    }).createRoute;
}
/**
 @deprecated It's no longer recommended to use the `FileRoute` class directly.
  Instead, use `createFileRoute('/path/to/file')(options)` to create a file route.
 */
class FileRoute {
    path;
    silent;
    constructor(path, _opts) {
        this.path = path;
        this.silent = _opts?.silent;
    }
    createRoute = (options) => {
        warning(this.silent, 'FileRoute is deprecated and will be removed in the next major version. Use the createFileRoute(path)(options) function instead.');
        const route = createRoute(options);
        route.isRoot = false;
        return route;
    };
}
/**
 @deprecated It's recommended not to split loaders into separate files.
  Instead, place the loader function in the the main route file, inside the
  `createFileRoute('/path/to/file)(options)` options.
 */
function FileRouteLoader(_path) {
    warning(false, `FileRouteLoader is deprecated and will be removed in the next major version. Please place the loader function in the the main route file, inside the \`createFileRoute('/path/to/file')(options)\` options`);
    return (loaderFn) => loaderFn;
}
class LazyRoute {
    options;
    constructor(opts) {
        this.options = opts;
    }
    match$ = (opts) => match$({ ...opts, from: this.options.id });
    match = (opts) => match({ ...opts, from: this.options.id });
    routeContext$ = (opts) => routeContext$({ ...opts, from: this.options.id });
    routeContext = (opts) => routeContext({ ...opts, from: this.options.id });
    search$ = (opts) => search$({ ...opts, from: this.options.id });
    search = (opts) => search({ ...opts, from: this.options.id });
    params$ = (opts) => params$({ ...opts, from: this.options.id });
    params = (opts) => params({ ...opts, from: this.options.id });
    loaderDeps$ = (opts) => loaderDeps$({ ...opts, from: this.options.id });
    loaderDeps = (opts) => loaderDeps({ ...opts, from: this.options.id });
    loaderData$ = (opts) => loaderData$({ ...opts, from: this.options.id });
    loaderData = (opts) => loaderData({ ...opts, from: this.options.id });
}
function createLazyRoute(id) {
    return (opts) => {
        return new LazyRoute({
            id: id,
            ...opts,
        });
    };
}
function createLazyFileRoute(id) {
    return (opts) => new LazyRoute({ id, ...opts });
}

class RouterRoot {
    router = input(injectRouter());
    options = input({});
    constructor() {
        const environmentInjector = inject(EnvironmentInjector);
        effect(() => {
            const [router, options] = [this.router(), this.options()];
            router.update({
                ...router.options,
                ...options,
                context: {
                    ...router.options.context,
                    ...options.context,
                    getRouteInjector(routeId, providers = []) {
                        return router.getRouteEnvInjector(routeId, environmentInjector, providers, router);
                    },
                },
            });
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouterRoot, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.3", type: RouterRoot, isStandalone: true, selector: "router-root,RouterRoot", inputs: { router: { classPropertyName: "router", publicName: "router", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, hostDirectives: [{ directive: Matches }], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouterRoot, decorators: [{
            type: Directive,
            args: [{
                    selector: 'router-root,RouterRoot',
                    hostDirectives: [Matches],
                }]
        }], ctorParameters: () => [] });

function canGoBack$({ injector } = {}) {
    !injector && assertInInjectionContext(canGoBack$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return routerState$({
            select: (s) => s.location.state.__TSR_index !== 0,
            injector,
        });
    });
}
function canGoBack({ injector } = {}) {
    !injector && assertInInjectionContext(canGoBack);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(canGoBack$({ injector }));
    });
}

function location$({ injector, select, } = {}) {
    !injector && assertInInjectionContext(location$);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return routerState$({
            injector,
            select: (state) => (select ? select(state.location) : state.location),
        });
    });
}
function location({ injector, select } = {}) {
    !injector && assertInInjectionContext(location);
    if (!injector) {
        injector = inject(Injector);
    }
    return runInInjectionContext(injector, () => {
        return toSignal(location$({ injector, select }));
    });
}

class RouterDevtools {
    injectedRouter = injectRouter();
    host = inject(ElementRef);
    ngZone = inject(NgZone);
    router = input(this.injectedRouter);
    initialIsOpen = input(undefined, { transform: booleanAttribute });
    panelOptions = input({});
    closeButtonOptions = input({});
    toggleButtonOptions = input({});
    shadowDOMTarget = input();
    containerElement = input();
    position = input();
    devtools = signal(null);
    constructor() {
        afterNextRender(() => {
            const router = untracked(this.router);
            const [initialIsOpen, panelOptions, closeButtonOptions, toggleButtonOptions, shadowDOMTarget, containerElement, position, activeRouterState,] = [
                untracked(this.initialIsOpen),
                untracked(this.panelOptions),
                untracked(this.closeButtonOptions),
                untracked(this.toggleButtonOptions),
                untracked(this.shadowDOMTarget),
                untracked(this.containerElement),
                untracked(this.position),
                router.state,
            ];
            // initial devTools
            this.devtools.set(new TanStackRouterDevtoolsCore({
                router,
                routerState: activeRouterState,
                initialIsOpen,
                position,
                panelProps: panelOptions,
                closeButtonProps: closeButtonOptions,
                toggleButtonProps: toggleButtonOptions,
                shadowDOMTarget,
                containerElement,
            }));
        });
        effect(() => {
            const devtools = this.devtools();
            if (!devtools)
                return;
            this.ngZone.runOutsideAngular(() => devtools.setRouter(this.router()));
        });
        effect((onCleanup) => {
            const devtools = this.devtools();
            if (!devtools)
                return;
            this.ngZone.runOutsideAngular(() => {
                const unsub = untracked(this.router).__store.subscribe(({ currentVal }) => {
                    devtools.setRouterState(currentVal);
                });
                onCleanup(() => unsub());
            });
        });
        effect(() => {
            const devtools = this.devtools();
            if (!devtools)
                return;
            this.ngZone.runOutsideAngular(() => {
                devtools.setOptions({
                    initialIsOpen: this.initialIsOpen(),
                    panelProps: this.panelOptions(),
                    closeButtonProps: this.closeButtonOptions(),
                    toggleButtonProps: this.toggleButtonOptions(),
                    position: this.position(),
                    containerElement: this.containerElement(),
                    shadowDOMTarget: this.shadowDOMTarget(),
                });
            });
        });
        effect((onCleanup) => {
            const devtools = this.devtools();
            if (!devtools)
                return;
            this.ngZone.runOutsideAngular(() => devtools.mount(this.host.nativeElement));
            onCleanup(() => {
                this.ngZone.runOutsideAngular(() => devtools.unmount());
            });
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouterDevtools, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.3", type: RouterDevtools, isStandalone: true, selector: "router-devtools,RouterDevtools", inputs: { router: { classPropertyName: "router", publicName: "router", isSignal: true, isRequired: false, transformFunction: null }, initialIsOpen: { classPropertyName: "initialIsOpen", publicName: "initialIsOpen", isSignal: true, isRequired: false, transformFunction: null }, panelOptions: { classPropertyName: "panelOptions", publicName: "panelOptions", isSignal: true, isRequired: false, transformFunction: null }, closeButtonOptions: { classPropertyName: "closeButtonOptions", publicName: "closeButtonOptions", isSignal: true, isRequired: false, transformFunction: null }, toggleButtonOptions: { classPropertyName: "toggleButtonOptions", publicName: "toggleButtonOptions", isSignal: true, isRequired: false, transformFunction: null }, shadowDOMTarget: { classPropertyName: "shadowDOMTarget", publicName: "shadowDOMTarget", isSignal: true, isRequired: false, transformFunction: null }, containerElement: { classPropertyName: "containerElement", publicName: "containerElement", isSignal: true, isRequired: false, transformFunction: null }, position: { classPropertyName: "position", publicName: "position", isSignal: true, isRequired: false, transformFunction: null } }, host: { styleAttribute: "display: block;" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: RouterDevtools, decorators: [{
            type: Directive,
            args: [{
                    selector: 'router-devtools,RouterDevtools',
                    host: { style: 'display: block;' },
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { DefaultError, DefaultNotFound, ERROR_COMPONENT_CONTEXT, LazyRoute, Link, MatchRoute, Matches, NOT_FOUND_COMPONENT_CONTEXT, NgRouter, NotFoundRoute, OnRendered, Outlet, ROUTER, ROUTER_STATE, RootRoute, Route, RouteApi, RouteMatch, RouterDevtools, RouterRoot, Transitioner, canGoBack, canGoBack$, childMatches, childMatches$, createFileRoute, createLazyFileRoute, createLazyRoute, createRootRoute, createRootRouteWithContext, createRoute, createRouter, injectRouter, injectRouterState, linkOptions, loaderData, loaderData$, loaderDeps, loaderDeps$, location, location$, match, match$, matchRoute, matchRoute$, matches, matches$, params, params$, parentMatches, parentMatches$, provideRouter, routeApi, routeContext, routeContext$, routerState, routerState$, search, search$ };
//# sourceMappingURL=tanstack-angular-router-experimental.mjs.map
