import { InjectionToken, Provider, type Type } from '@angular/core';
import { AnyContext, AnyRoute, AnyRouter, BaseRootRoute, BaseRoute, BaseRouteApi, ConstrainLiteral, ErrorComponentProps, NotFoundRouteProps, RegisteredRouter, ResolveFullPath, ResolveId, ResolveParams, RootRouteId, RootRouteOptions, RouteConstraints, RouteIds, RouteOptions } from '@tanstack/router-core';
import { LoaderDataRoute } from './loader-data';
import { LoaderDepsRoute } from './loader-deps';
import { MatchRoute } from './match';
import { ParamsRoute } from './params';
import { RouteContextRoute } from './route-context';
import { SearchRoute } from './search';
declare module '@tanstack/router-core' {
    interface UpdatableRouteOptionsExtensions {
        component?: () => RouteComponent;
        errorComponent?: false | null | (() => RouteComponent);
        notFoundComponent?: () => RouteComponent;
        pendingComponent?: () => RouteComponent;
        providers?: Provider[];
    }
    interface RouteExtensions<TId extends string, TFullPath extends string> {
        match$: MatchRoute<true, TId>;
        match: MatchRoute<false, TId>;
        routeContext$: RouteContextRoute<true, TId>;
        routeContext: RouteContextRoute<false, TId>;
        search$: SearchRoute<true, TId>;
        search: SearchRoute<false, TId>;
        params$: ParamsRoute<true, TId>;
        params: ParamsRoute<false, TId>;
        loaderDeps$: LoaderDepsRoute<true, TId>;
        loaderDeps: LoaderDepsRoute<false, TId>;
        loaderData$: LoaderDataRoute<true, TId>;
        loaderData: LoaderDataRoute<false, TId>;
    }
}
export declare const ERROR_COMPONENT_CONTEXT: InjectionToken<ErrorComponentProps>;
export declare const NOT_FOUND_COMPONENT_CONTEXT: InjectionToken<NotFoundRouteProps>;
export type RouteComponent<TComponent extends object = object> = Type<TComponent>;
export declare function routeApi<const TId, TRouter extends AnyRouter = RegisteredRouter>(id: ConstrainLiteral<TId, RouteIds<TRouter['routeTree']>>): RouteApi<TId, TRouter>;
export declare class RouteApi<TId, TRouter extends AnyRouter = RegisteredRouter> extends BaseRouteApi<TId, TRouter> {
    /**
     * @deprecated Use the `getRouteApi` function instead.
     */
    constructor({ id }: {
        id: TId;
    });
    match$: MatchRoute<true, TId>;
    match: MatchRoute<false, TId>;
    routeContext$: RouteContextRoute<true, TId>;
    routeContext: RouteContextRoute<false, TId>;
    search$: SearchRoute<true, TId>;
    search: SearchRoute<false, TId>;
    params$: ParamsRoute<true, TId>;
    params: ParamsRoute<false, TId>;
    loaderDeps$: LoaderDepsRoute<true, TId>;
    loaderDeps: LoaderDepsRoute<false, TId>;
    loaderData$: LoaderDataRoute<true, TId>;
    loaderData: LoaderDataRoute<false, TId>;
}
export declare class Route<in out TParentRoute extends RouteConstraints['TParentRoute'] = AnyRoute, in out TPath extends RouteConstraints['TPath'] = '/', in out TFullPath extends RouteConstraints['TFullPath'] = ResolveFullPath<TParentRoute, TPath>, in out TCustomId extends RouteConstraints['TCustomId'] = string, in out TId extends RouteConstraints['TId'] = ResolveId<TParentRoute, TCustomId, TPath>, in out TSearchValidator = undefined, in out TParams = ResolveParams<TPath>, in out TRouterContext = AnyContext, in out TRouteContextFn = AnyContext, in out TBeforeLoadFn = AnyContext, in out TLoaderDeps extends Record<string, any> = {}, in out TLoaderFn = undefined, in out TChildren = unknown, in out TFileRouteTypes = unknown> extends BaseRoute<TParentRoute, TPath, TFullPath, TCustomId, TId, TSearchValidator, TParams, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, TChildren, TFileRouteTypes> {
    /**
     * @deprecated Use the `createRoute` function instead.
     */
    constructor(options?: RouteOptions<TParentRoute, TId, TCustomId, TFullPath, TPath, TSearchValidator, TParams, TLoaderDeps, TLoaderFn, TRouterContext, TRouteContextFn, TBeforeLoadFn>);
    match$: MatchRoute<true, TId>;
    match: MatchRoute<false, TId>;
    routeContext$: RouteContextRoute<true, TId>;
    routeContext: RouteContextRoute<false, TId>;
    search$: SearchRoute<true, TId>;
    search: SearchRoute<false, TId>;
    params$: ParamsRoute<true, TId>;
    params: ParamsRoute<false, TId>;
    loaderDeps$: LoaderDepsRoute<true, TId>;
    loaderDeps: LoaderDepsRoute<false, TId>;
    loaderData$: LoaderDataRoute<true, TId>;
    loaderData: LoaderDataRoute<false, TId>;
}
export declare function createRoute<TParentRoute extends RouteConstraints['TParentRoute'] = AnyRoute, TPath extends RouteConstraints['TPath'] = '/', TFullPath extends RouteConstraints['TFullPath'] = ResolveFullPath<TParentRoute, TPath>, TCustomId extends RouteConstraints['TCustomId'] = string, TId extends RouteConstraints['TId'] = ResolveId<TParentRoute, TCustomId, TPath>, TSearchValidator = undefined, TParams = ResolveParams<TPath>, TRouteContextFn = AnyContext, TBeforeLoadFn = AnyContext, TLoaderDeps extends Record<string, any> = {}, TLoaderFn = undefined, TChildren = unknown>(options: RouteOptions<TParentRoute, TId, TCustomId, TFullPath, TPath, TSearchValidator, TParams, TLoaderDeps, TLoaderFn, AnyContext, TRouteContextFn, TBeforeLoadFn>): Route<TParentRoute, TPath, TFullPath, TCustomId, TId, TSearchValidator, TParams, AnyContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, TChildren, unknown>;
export type AnyRootRoute = RootRoute<any, any, any, any, any, any, any, any>;
export declare function createRootRouteWithContext<TRouterContext extends {}>(): <TRouteContextFn = AnyContext, TBeforeLoadFn = AnyContext, TSearchValidator = undefined, TLoaderDeps extends Record<string, any> = {}, TLoaderFn = undefined>(options?: RootRouteOptions<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn>) => RootRoute<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, unknown, unknown>;
export declare class RootRoute<in out TSearchValidator = undefined, in out TRouterContext = {}, in out TRouteContextFn = AnyContext, in out TBeforeLoadFn = AnyContext, in out TLoaderDeps extends Record<string, any> = {}, in out TLoaderFn = undefined, in out TChildren = unknown, in out TFileRouteTypes = unknown> extends BaseRootRoute<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, TChildren, TFileRouteTypes> {
    /**
     * @deprecated `RootRoute` is now an internal implementation detail. Use `createRootRoute()` instead.
     */
    constructor(options?: RootRouteOptions<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn>);
    match$: MatchRoute<true, RootRouteId>;
    match: MatchRoute<false, RootRouteId>;
    routeContext$: RouteContextRoute<true, RootRouteId>;
    routeContext: RouteContextRoute<false, RootRouteId>;
    search$: SearchRoute<true, RootRouteId>;
    search: SearchRoute<false, RootRouteId>;
    params$: ParamsRoute<true, RootRouteId>;
    params: ParamsRoute<false, RootRouteId>;
    loaderDeps$: LoaderDepsRoute<true, RootRouteId>;
    loaderDeps: LoaderDepsRoute<false, RootRouteId>;
    loaderData$: LoaderDataRoute<true, RootRouteId>;
    loaderData: LoaderDataRoute<false, RootRouteId>;
}
export declare function createRootRoute<TSearchValidator = undefined, TRouterContext = {}, TRouteContextFn = AnyContext, TBeforeLoadFn = AnyContext, TLoaderDeps extends Record<string, any> = {}, TLoaderFn = undefined>(options?: RootRouteOptions<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn>): RootRoute<TSearchValidator, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, unknown, unknown>;
export declare class NotFoundRoute<TParentRoute extends AnyRootRoute, TRouterContext = AnyContext, TRouteContextFn = AnyContext, TBeforeLoadFn = AnyContext, TSearchValidator = undefined, TLoaderDeps extends Record<string, any> = {}, TLoaderFn = undefined, TChildren = unknown> extends Route<TParentRoute, '/404', '/404', '404', '404', TSearchValidator, {}, TRouterContext, TRouteContextFn, TBeforeLoadFn, TLoaderDeps, TLoaderFn, TChildren> {
    constructor(options: Omit<RouteOptions<TParentRoute, string, string, string, string, TSearchValidator, {}, TLoaderDeps, TLoaderFn, TRouterContext, TRouteContextFn, TBeforeLoadFn>, 'caseSensitive' | 'parseParams' | 'stringifyParams' | 'path' | 'id' | 'params'>);
}
